# -*- coding: utf-8 -*-
"""Make sure that the parsing functions for specific functions are imported properly."""

from ioc_finder import parse_urls


def test_url_parsing_func():
    results = parse_urls('https://google.com')
    assert results == ['https://google.com']



# -*- coding: utf-8 -*-

from ioc_finder import find_iocs


def test_leading_zero():
    """Sections of IP addresses that start with a leading zero should be interpreted as being in base 8."""
    s = '0177.0.0.01'
    results = find_iocs(s)
    assert results['ipv4s'] == ['127.0.0.1']

    s = '226.000.000.037'
    results = find_iocs(s)
    assert results['ipv4s'] == ['226.0.0.31']

    s = '014.0.0.01'
    results = find_iocs(s)
    assert results['ipv4s'] == ['12.0.0.1']

    # because `018` (the first section of the ip below) is not a valid octal number, it should not be converted
    s = '018.0.0.01'
    results = find_iocs(s)
    assert results['ipv4s'] == ['18.0.0.1']


def test_a_b_c_format():
    results = find_iocs('111.111.1111')
    assert results['ipv4s'] == ['111.111.4.87']

    results = find_iocs('10.0.514')
    assert results['ipv4s'] == ['10.0.2.2']

    # here is some code to show how sections c and d of dotted decimal form are calculated from the given examples:
    # x = 1111
    # print(f'{x // 256}.{(x - 256) % 256}')


def test_a_b_format():
    results = find_iocs('1.300')
    assert results['ipv4s'] == ['1.0.1.44']

    results = find_iocs('1.256')
    assert results['ipv4s'] == ['1.0.1.0']

    results = find_iocs('1.15')
    assert results['ipv4s'] == ['1.0.0.15']

    results = find_iocs('1.65793')
    assert results['ipv4s'] == ['1.1.1.1']

    results = find_iocs('1.67794')
    assert results['ipv4s'] == ['1.1.8.210']


def test_numeric_forms():
    # these examples are from: https://bugzilla.mozilla.org/show_bug.cgi?id=67730

    s = 'http://3486011863'
    results = find_iocs(s)
    assert results['ipv4s'] == ['207.200.81.215']

    s = 'http://00000000317.00000000310.00000000121.00000000327/'
    results = find_iocs(s)
    assert results['ipv4s'] == ['207.200.81.215']

    s = 'http://4294967503.4294967496.4294967377.4294967511/'
    results = find_iocs(s)
    assert results['ipv4s'] == ['207.200.81.215']


def test_real_obfuscated_forms():
    # these examples come from: https://securelist.com/new-brazilian-banking-trojans-recycle-old-url-obfuscation-tricks/29558/

    s = 'http://0x42.0x66.0x0d.0x63'
    results = find_iocs(s)
    assert results['ipv4s'] == ['66.102.13.99']

    s = 'http://0x42660d63'
    results = find_iocs(s)
    assert results['ipv4s'] == ['66.102.13.99']

    s = 'http://1113984355'
    results = find_iocs(s)
    assert results['ipv4s'] == ['66.102.13.99']

    s = 'http://00000102.00000146.00000015.00000143'
    results = find_iocs(s)
    assert results['ipv4s'] == ['66.102.13.99']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .ioc_finder import (
    find_iocs,
    parse_urls,
    parse_domain_names,
    parse_ipv4_addresses,
    parse_ipv6_addresses,
    parse_complete_email_addresses,
    parse_email_addresses,
    parse_imphashes_,
    parse_authentihashes_,
    parse_md5s,
    parse_sha1s,
    parse_sha256s,
    parse_sha512s,
    parse_ssdeeps,
    parse_asns,
    parse_cves,
    parse_ipv4_cidrs,
    parse_registry_key_paths,
    parse_google_adsense_ids,
    parse_google_analytics_ids,
    parse_bitcoin_addresses,
    parse_xmpp_addresses,
    parse_mac_addresses,
    parse_user_agents,
    parse_file_paths,
    parse_phone_numbers,
    parse_pre_attack_tactics,
    parse_pre_attack_techniques,
    parse_enterprise_attack_tactics,
    parse_enterprise_attack_techniques,
    parse_mobile_attack_tactics,
    parse_mobile_attack_techniques,
    parse_tlp_labels,
)

__author__ = '''Floyd Hightower'''
__version__ = '4.0.2'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pytest

from ioc_finder import find_iocs


@pytest.fixture
def text_a():
    """Provide some generic text for the tests below."""
    return 'example.com is a nice domain if you consider http://bad.com/test/bingo.php to be bad. {} {} {} 1.2.3.4 192.64.55.61 bad12312@example.org'.format(
        'a' * 32, 'b' * 40, 'c' * 64
    )


def test_domain_name_with_underscore():
    # see https://github.com/fhightower/ioc-finder/issues/26
    s = 'o_o.lgms.nl'
    results = find_iocs(s)
    assert results['domains'] == ['o_o.lgms.nl']

    s = '_jabber._tcp.gmail.com'
    results = find_iocs(s)
    assert results['domains'] == ['_jabber._tcp.gmail.com']


def test_domain_name_with_underscore_preceeded_by_markdown_italics():
    s = '_o_o.lgms.nl_'
    results = find_iocs(s)
    assert results['domains'] == ['o_o.lgms.nl']


def test_url_with_underscore_in_subdomain():
    # see https://github.com/fhightower/ioc-finder/issues/26
    s = 'https://o_o.lgms.nl/'
    results = find_iocs(s)
    assert results['urls'] == ['https://o_o.lgms.nl/']


def test_ioc_finder(text_a):
    iocs = find_iocs(text_a)
    assert len(iocs['domains']) == 3
    assert 'example.com' in iocs['domains']
    assert 'example.org' in iocs['domains']
    assert 'bad.com' in iocs['domains']

    assert iocs['email_addresses_complete'] == ['bad12312@example.org']

    assert len(iocs['ipv4s']) == 2
    assert '1.2.3.4' in iocs['ipv4s']
    assert '192.64.55.61' in iocs['ipv4s']

    assert iocs['urls'] == ['http://bad.com/test/bingo.php']
    assert iocs['md5s'] == ['aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa']
    assert iocs['sha1s'] == ['bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb']
    assert iocs['sha256s'] == ['cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc']


def test_url_parsing():
    """Test some specific url examples."""
    s = "https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh']

    s = '<link href="http://fonts.googleapis.com/css?family=Lato:400,700" rel="stylesheet" type="text/css"/>'
    iocs = find_iocs(s)
    assert 'http://fonts.googleapis.com/css?family=Lato:400,700' in iocs['urls']

    s = '<link href="http://fonts.googleapis.com/css?family=Lato:400,700" rel="stylesheet" type="text/css"/>'
    iocs = find_iocs(s)
    assert 'http://fonts.googleapis.com/css?family=Lato:400,700' in iocs['urls']

    s = '<a href="https://bit.ly/12345#abcd" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results['urls'] == ['https://bit.ly/12345#abcd']

    s = '<a href="https://bit.ly/12345" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results['urls'] == ['https://bit.ly/12345']

    s = '<a href="https://bit.ly" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results['urls'] == ['https://bit.ly']

    s = '<a href="https://bit.ly/" target="_blank" style="text-decoration:none;;">'
    results = find_iocs(s)
    assert results['urls'] == ['https://bit.ly/']

    s = 'http://example.com//test'
    results = find_iocs(s)
    assert results['urls'] == ['http://example.com//test']


def test_issue_45_url_parsing():
    s = 'http://wmfolcs3.pn.4y.nv.kr2x1dt.net/gz+/(y%40%26//%3c7aew%5cqv%0a/%0bcz,r/r%5c%7b/7re//6%3e/f%23%7ce0p\'6_%09/d%5c'
    results = find_iocs(s)
    assert results['urls'] == [
        'http://wmfolcs3.pn.4y.nv.kr2x1dt.net/gz+/(y%40%26//%3c7aew%5cqv%0a/%0bcz,r/r%5c%7b/7re//6%3e/f%23%7ce0p\'6_%09/d%5c'
    ]


def test_schemeless_url_parsing():
    """Test parsing URLs without a scheme."""
    s = "github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh"
    iocs = find_iocs(s)
    assert iocs['urls'] == ['github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh']

    s = 'github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh hightower.space/projects'
    iocs = find_iocs(s)
    assert len(iocs['urls']) == 2
    assert 'hightower.space/projects' in iocs['urls']
    assert 'github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh' in iocs['urls']

    s = 'https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh hightower.space/projects'
    iocs = find_iocs(s, parse_urls_without_scheme=False)
    assert iocs['urls'] == ['https://github.com/StylishThemes/GitHub-Dark/blob/master/tools/authors.sh']


def test_address_email_address():
    s = ">test@[192.168.2.1]<"
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['test@[192.168.2.1]']
    assert iocs['email_addresses'] == ['test@[192.168.2.1]']
    assert iocs['ipv4s'] == ['192.168.2.1']

    s = "bad@[192.168.7.3]"
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['192.168.7.3']
    assert iocs['email_addresses_complete'] == ['bad@[192.168.7.3]']
    assert iocs['email_addresses'] == ['bad@[192.168.7.3]']

    s = "bad@[192.168.7.3]aaaaa"
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['bad@[192.168.7.3]']
    assert iocs['email_addresses'] == ['bad@[192.168.7.3]']

    s = "jsmith@[IPv6:2001:db8::1]"
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['jsmith@[IPv6:2001:db8::1]']
    assert iocs['email_addresses'] == ['jsmith@[IPv6:2001:db8::1]']
    assert iocs['ipv6s'] == ['2001:db8::1']


def test_address_domain_url():
    s = "http://192.64.55.61/test.php"
    iocs = find_iocs(s)
    assert iocs['urls'] == ['http://192.64.55.61/test.php']
    assert iocs['ipv4s'] == ['192.64.55.61']


def test_url_domain_name_parsing():
    s = "http://foo.youtube/test.php"
    iocs = find_iocs(s)
    assert iocs['urls'] == ['http://foo.youtube/test.php']
    assert iocs['domains'] == ['foo.youtube']


def test_unicode_domain_name():
    s = "ȩxample.com"
    iocs = find_iocs(s)
    assert iocs['domains'] == ['\\u0229xample.com']


def test_ioc_deduplication():
    """Make sure the results returned from the ioc_finder are deduplicated."""
    iocs = find_iocs('example.com example.com')
    assert iocs['domains'] == ['example.com']


def test_file_hash_order():
    s = "{} {}".format('a' * 32, 'b' * 40)
    iocs = find_iocs(s)
    assert iocs['md5s'][0] == 'a' * 32
    assert iocs['sha1s'][0] == 'b' * 40


def test_file_hash_parsing():
    s = 'this is a test{}'.format('a' * 32)
    iocs = find_iocs(s)
    assert iocs['md5s'] == []

    s = 'this is a test {}'.format('a' * 32)
    iocs = find_iocs(s)
    assert iocs['md5s'] == ['a' * 32]

    s = 'this is a test "{}"'.format('a' * 32)
    iocs = find_iocs(s)
    assert iocs['md5s'] == ['a' * 32]

    s = 'this is a test {}.'.format('a' * 32)
    iocs = find_iocs(s)
    assert iocs['md5s'] == ['a' * 32]

    s = '0x1a1db93766e31994507511c9c70a1dd94465cf6d'
    iocs = find_iocs(s)
    assert iocs['sha1s'] == ['1a1db93766e31994507511c9c70a1dd94465cf6d']


def test_url_boundaries():
    """Make sure the boundaries for a url are correct."""
    s = """http://192.168.0.1/test/bad.html</a><br></div>"""
    iocs = find_iocs(s)
    assert iocs['urls'] == ['http://192.168.0.1/test/bad.html']

    s = """<IMg SRc="https://i.imgur.com/abc.png#4827766048"/><br>
<IMg SRc="https://i.imgur.com/def.png#4827766048"/><br>"""
    iocs = find_iocs(s)
    assert 'https://i.imgur.com/abc.png#4827766048' in iocs['urls']
    assert 'https://i.imgur.com/def.png#4827766048' in iocs['urls']
    assert len(iocs['urls']) == 2

    s = """<IMg SRc="https://i.imgur.com/abc.png"/><br>"""
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://i.imgur.com/abc.png']

    s = """(https://i.imgur.com/abc.png)"""
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://i.imgur.com/abc.png']

    s = """(https://i.imgur.com/abc.png#abc)"""
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://i.imgur.com/abc.png#abc']

    s = """[https://i.imgur.com/abc.png](https://i.imgur.com/abc.png)"""
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://i.imgur.com/abc.png']

    s = """[https://i.imgur.com/abc.png#abc](https://i.imgur.com/abc.png#abc)"""
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://i.imgur.com/abc.png#abc']

    s = "DownloadString('https://hacks4all[.]net/rdp.ps1');g $I"
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://hacks4all.net/rdp.ps1']

    s = 'DownloadString("https://hacks4all[.]net/rdp.ps1");g $I'
    iocs = find_iocs(s)
    assert iocs['urls'] == ['https://hacks4all.net/rdp.ps1']


def test_domain_parsing():
    s = "Host: dfasdfa (mz-fcb301p.ocn.ad.jp asdfsdafs"
    iocs = find_iocs(s)
    assert iocs['domains'] == ['mz-fcb301p.ocn.ad.jp']

    s = "smtp.mailfrom"
    iocs = find_iocs(s)
    assert iocs['domains'] == []

    s = "bar.com"
    iocs = find_iocs(s)
    assert iocs['domains'] == ['bar.com']

    s = 'bar.com"'
    iocs = find_iocs(s)
    assert iocs['domains'] == ['bar.com']

    s = "bar.com'"
    iocs = find_iocs(s)
    assert iocs['domains'] == ['bar.com']

    s = "bar.com."
    iocs = find_iocs(s)
    assert iocs['domains'] == ['bar.com']

    # make sure domains of different casings are properly parsed: https://github.com/fhightower/ioc-finder/issues/47
    iocs = find_iocs("BAR.com")
    assert iocs['domains'] == ['bar.com']
    iocs = find_iocs("bar.COM")
    assert iocs['domains'] == ['bar.com']
    iocs = find_iocs("BAR.COM")
    assert iocs['domains'] == ['bar.com']


def test_email_address_parsing():
    s = 'my email is: foo"bar@gmail.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['foo"bar@gmail.com']
    assert iocs['email_addresses'] == ['bar@gmail.com']

    s = 'Abc\\@def@example.com'
    iocs = find_iocs(s)
    print(iocs['email_addresses_complete'])
    print(iocs['email_addresses'])
    assert iocs['email_addresses_complete'] == ['Abc\\@def@example.com']
    assert iocs['email_addresses'] == ['def@example.com']

    s = 'foobar@gmail.com"'
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['foobar@gmail.com']
    assert iocs['email_addresses'] == ['foobar@gmail.com']

    s = 'foobar@gmail.comahhhhhhhh'
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == []
    assert iocs['email_addresses'] == []

    s = '"foobar@gmail.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['"foobar@gmail.com']
    assert iocs['email_addresses'] == ['foobar@gmail.com']

    s = 'smtp.mailfrom=example@example.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['smtp.mailfrom=example@example.com']
    assert iocs['email_addresses'] == ['example@example.com']

    s = '"foo@bar.com"'
    iocs = find_iocs(s)
    assert iocs['email_addresses_complete'] == ['"foo@bar.com']
    assert iocs['email_addresses'] == ['foo@bar.com']

    # making sure that the `parse_domain_from_email_address` argument is working properly
    s = 'foo@bar.com.'
    iocs = find_iocs(s, parse_domain_from_email_address=False)
    assert iocs['email_addresses'] == ['foo@bar.com']
    assert iocs['domains'] == []

    s = '"foo@bar.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses'] == ['foo@bar.com']

    # validating https://github.com/fhightower/ioc-finder/issues/40 is fixed
    s = '-----foo@bar.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses'] == ['foo@bar.com']

    s = 'foo-burt@bar.com f-1@bar.com'
    iocs = find_iocs(s)
    assert len(iocs['email_addresses']) == 2
    assert 'foo-burt@bar.com' in iocs['email_addresses']
    assert 'f-1@bar.com' in iocs['email_addresses']


def test_erroneous_ip_address_parsing():
    # the two tests below make sure that IP addresses are not parsed from strings with decimals in them
    s = '2018.12.15.14.05.43'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '111.12.15.14.05.43'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '.18.12.15.14'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '18.12.15.1411111111'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '018.12.15.14'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['18.12.15.14']

    s = '18.12.15.14.'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['18.12.15.14']

    # the three tests below make sure that IP addresses are not parsed from sequences with large numbers in them
    s = '1112.15.14.05'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '15.1112.14.05'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '15.14.05.1112'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []


def test_ip_address_systematically():
    # TODO: for many of the assertions below, I would like to be more explicit; I would like to change tests like `len(iocs['ipv4s']) == 1` to `iocs['ipv4s'] == ['1.1.1.1']`
    s = '1.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['1.1.1.1']

    s = '.1.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    # I would like to match in this situation to capture ip address that are at the end of a sentence
    s = '1.1.1.1.'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['1.1.1.1']

    s = '.1.1.1.1.'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '1.1.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '.1.1.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '1.1.1.1.1.'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '.1.1.1.1.1.'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '1.1.1.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '1.1.1.1.a'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []

    s = '1.01.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['1.1.1.1']

    s = '01.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['1.1.1.1']

    s = '01.01.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == ['1.1.1.1']

    s = '0001.1.1.1'
    iocs = find_iocs(s)
    assert iocs['ipv4s'] == []


def test_asn_parsing():
    s = 'NWD2HUBCAS8.ad.analog.com'
    iocs = find_iocs(s)
    assert iocs['asns'] == []

    s = 'here is an asn: "AS8"'
    iocs = find_iocs(s)
    assert iocs['asns'] == ['ASN8']

    s = 'here is an asn: AS8foobar'
    iocs = find_iocs(s)
    assert iocs['asns'] == []

    s = 'as8'
    iocs = find_iocs(s)
    assert iocs['asns'] == ['ASN8']


def test_onion_parsing():
    s = 'foo.onion'
    iocs = find_iocs(s)
    assert iocs['domains'] == ['foo.onion']

    s = 'http://foo.onion/test'
    iocs = find_iocs(s)
    assert iocs['urls'] == ['http://foo.onion/test']
    assert iocs['domains'] == ['foo.onion']


def test_deduplication_of_indicators_with_different_cases():
    s = 'example.com Example.com exAmplE.com'
    iocs = find_iocs(s)
    assert iocs['domains'] == ['example.com']

    s = 'bad@example.com bad@Example.com bad@exAmplE.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses'] == ['bad@example.com']

    s = 'bad@example.com Bad@example.com'
    iocs = find_iocs(s)
    assert iocs['email_addresses'] == ['bad@example.com']

    s = 'http://example.com/test http://EXAMple.com/test'
    iocs = find_iocs(s)
    assert iocs['urls'] == ['http://example.com/test']

    s = 'http://example.com/test http://EXAMple.com/TEST'
    iocs = find_iocs(s)
    assert len(iocs['urls']) == 2
    assert 'http://example.com/test' in iocs['urls']
    assert 'http://example.com/TEST' in iocs['urls']


def test_google_adsense_publisher_ids():
    s = "PUB-1234567891234567"
    iocs = find_iocs(s)
    assert iocs['google_adsense_publisher_ids'] == ['pub-1234567891234567']

    s = "pUb-1234567891234567"
    iocs = find_iocs(s)
    assert iocs['google_adsense_publisher_ids'] == []


def test_google_analyitics_tracker_ids():
    s = "ua-000000-1"
    iocs = find_iocs(s)
    assert iocs['google_analytics_tracker_ids'] == ['UA-000000-1']


def test_google_casing_deduplication():
    s = "pub-1234567891234567 PUB-1234567891234567 pUb-1234567891234567"
    iocs = find_iocs(s)
    assert iocs['google_adsense_publisher_ids'] == ['pub-1234567891234567']

    s = "UA-000000-1 ua-000000-1"
    iocs = find_iocs(s)
    assert iocs['google_analytics_tracker_ids'] == ['UA-000000-1']


def test_not_parsing_imphash():
    s = 'imphash 18ddf28a71089acdbab5038f58044c0a'
    iocs = find_iocs(s, parse_imphashes=False)
    assert 'imphashes' not in iocs
    assert iocs['md5s'] == ['18ddf28a71089acdbab5038f58044c0a']


def test_not_parsing_authentihash():
    s = 'authentihash 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4'
    iocs = find_iocs(s, parse_authentihashes=False)
    assert 'authentihashes' not in iocs
    assert iocs['sha256s'] == ['3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4']


def test_mac_address_parsing():
    s = '2019.02.15'
    iocs = find_iocs(s)
    assert iocs['mac_addresses'] == []


def test_windows_file_paths():
    s = r'test C:\Users\<username>\AppData\Roaming\Macromedia\Flash Player\macromedia\bin is a bad file path'
    iocs = find_iocs(s)
    assert iocs['file_paths'] == [r'C:\Users\<username>\AppData\Roaming\Macromedia\Flash Player\macromedia\bin']

    s = r'test C:\Users\<username>\AppData\Roaming\Macromedia\Flash Player\macromedia\bin\ is a bad file path'
    iocs = find_iocs(s)
    assert iocs['file_paths'] == [
        'C:\\Users\\<username>\\AppData\\Roaming\\Macromedia\\Flash Player\\macromedia\\bin\\'
    ]


def test_unix_file_paths():
    # https://github.com/fhightower/ioc-finder/issues/42
    s = 'https://twitter.com/'
    iocs = find_iocs(s)
    assert iocs['file_paths'] == []

    s = r'test /Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e file'
    iocs = find_iocs(s)
    assert iocs['file_paths'] == [r'/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e']

    s = r'test /Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/ file'
    iocs = find_iocs(s)
    assert iocs['file_paths'] == [r'/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/']


def test_phone_number_parsing():
    # https://github.com/fhightower/ioc-finder/issues/43
    s = '3-2-1 0.02'
    iocs = find_iocs(s)
    assert iocs['phone_numbers'] == []


def test_ipv6_parsing():
    # https://github.com/fhightower/ioc-finder/issues/37
    s = '11:04:10 -0500'
    iocs = find_iocs(s)
    assert iocs['ipv6s'] == []


def test_ssdeep_parsing():
    # https://github.com/fhightower/ioc-finder/issues/36
    s = '11:04:10 -0500'
    iocs = find_iocs(s)
    assert iocs['ssdeeps'] == []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Using hypothesis (https://hypothesis.readthedocs.io/en/latest/index.html) to test the finder."""

from hypothesis import given, settings
from hypothesis.provisional import domains, urls, ip4_addr_strings, ip6_addr_strings

from ioc_finder import find_iocs


@given(urls())
@settings(deadline=None)
def test_url_parsing(url):
    url = url.lower()
    iocs = find_iocs(url)
    failure = False

    try:
        assert len(iocs['urls']) == 1
        assert iocs['urls'][0] == url
    except AssertionError as e:
        failure = True
        print('Failed on url: {}'.format(url))

    if failure:
        raise AssertionError('Error parsing urls')


@given(domains())
@settings(deadline=None)
def test_domain_parsing(domain):
    domain = domain.lower()
    iocs = find_iocs(domain)
    failure = False

    try:
        assert len(iocs['domains']) == 1
        assert iocs['domains'][0] == domain
    except AssertionError as e:
        failure = True
        print('Failed on domain: {}'.format(domain))

    if failure:
        raise AssertionError('Error parsing domains')


@given(ip4_addr_strings())
@settings(deadline=None)
def test_ipv4_parsing(ipv4):
    iocs = find_iocs(ipv4)
    failure = False

    try:
        assert len(iocs['ipv4s']) == 1
        assert iocs['ipv4s'][0] == ipv4
    except AssertionError as e:
        failure = True
        print('Failed on ipv4: {}'.format(ipv4))

    if failure:
        raise AssertionError('Error parsing ipv4s')


@given(ip6_addr_strings())
@settings(deadline=None)
def test_ipv6_parsing(ipv6):
    iocs = find_iocs(ipv6)
    failure = False

    try:
        assert len(iocs['ipv6s']) == 1
        assert iocs['ipv6s'][0] == ipv6
    except AssertionError as e:
        failure = True
        print('Failed on ipv6: {}'.format(ipv6))

    if failure:
        raise AssertionError('Error parsing ipv6s')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Python package for finding observables in text."""

import concurrent.futures
import json
import os
import sys
from typing import Dict, List

import click
import ioc_fanger
from pyparsing import ParseResults

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), ".")))
import ioc_grammars


def _deduplicate(indicator_list: List) -> List:
    """Deduplicate the list of observables."""
    return list(set(indicator_list))


def _listify(indicator_list: ParseResults) -> List:
    """Convert the multi-dimensional indicator list into a one-dimensional indicator list with empty entries and duplicates removed."""
    return _deduplicate([indicator[0] for indicator in indicator_list if indicator[0]])


def _remove_items(items: List[str], text: str) -> str:
    """Remove each item from the text."""
    for item in items:
        text = text.replace(item, ' ')
    return text


def _prepare_text(text: str) -> str:
    """Fang (https://ioc-fang.hightower.space/) and encode the text in such a way that all Unicode domain names are converted into their punycode representation."""
    text = ioc_fanger.fang(text)
    # text = text.encode('idna').decode('utf-8')
    return text


# TODO: replace this function with the democritus.stringRemoveFromEnd
def _remove_from_end(input_string: str, string_to_remove: str) -> str:
    """."""
    if input_string.endswith(string_to_remove):
        desired_string_final_index = len(input_string) - len(string_to_remove)
        updated_string = input_string[:desired_string_final_index]
        return updated_string
    else:
        return input_string


def parse_urls(text: str, parse_urls_without_scheme: bool = True) -> List:
    """."""
    if parse_urls_without_scheme:
        urls = ioc_grammars.scheme_less_url.searchString(text)
    else:
        urls = ioc_grammars.url.searchString(text)
    urls = _listify(urls)

    clean_urls = []

    # clean the url
    for url in urls:
        # remove `"` and `'` characters from the end of a URL
        url = url.rstrip('"').rstrip("'")

        # remove a final ')' if there is a '(' in the url
        if url.endswith(')') and '(' not in url:
            url = url.rstrip(')')

        # remove `'/>` and `"/>` from the end of a URL (this character string occurs at the end of an HMTL tag with )
        url = _remove_from_end(url, '\'/>')
        url = _remove_from_end(url, '"/>')

        clean_urls.append(url)

    # return the cleaned urls - I deduplicate them again because the structure of the URL may have changed when it was cleaned
    return _deduplicate(clean_urls)


def _remove_url_paths(urls: List, text: str) -> str:
    """Remove the path from each url from the text."""
    for url in urls:
        parsed_url = ioc_grammars.scheme_less_url.parseString(url)
        url_path = parsed_url.url_path

        is_cidr_range = parse_ipv4_cidrs(str(url))
        # if the 'url' has a URL path and is not a cidr range, remove the url_path
        if not is_cidr_range and len(url_path) > 1:
            text = text.replace(url_path, ' ')
    return text


def parse_domain_names(text):
    """."""
    domains = ioc_grammars.domain_name.searchString(text.lower())
    return _listify(domains)


def parse_ipv4_addresses(text):
    """."""
    addresses = ioc_grammars.ipv4_address.searchString(text)
    return _listify(addresses)


def parse_ipv6_addresses(text):
    """."""
    addresses = ioc_grammars.ipv6_address.searchString(text)
    return _listify(addresses)


def parse_complete_email_addresses(text: str) -> List:
    """."""
    email_addresses = ioc_grammars.complete_email_address.searchString(text)
    return _listify(email_addresses)


def parse_email_addresses(text: str) -> List:
    """."""
    email_addresses = ioc_grammars.email_address.searchString(text)
    return _listify(email_addresses)


# there is a trailing underscore on this function to differentiate it from the argument with the same name
def parse_imphashes_(text: str) -> List:
    """."""
    full_imphash_instances = ioc_grammars.imphash.searchString(text.lower())
    full_imphash_instances = _listify(full_imphash_instances)

    imphashes = []

    for imphash in full_imphash_instances:
        imphashes.append(ioc_grammars.imphash.parseString(imphash).hash[0])

    return imphashes


# there is a trailing underscore on this function to differentiate it from the argument with the same name
def parse_authentihashes_(text: str) -> List:
    """."""
    full_authentihash_instances = ioc_grammars.authentihash.searchString(text.lower())
    full_authentihash_instances = _listify(full_authentihash_instances)

    authentihashes = []

    for authentihash in full_authentihash_instances:
        authentihashes.append(ioc_grammars.authentihash.parseString(authentihash).hash[0])

    return authentihashes


def parse_md5s(text):
    """."""
    md5s = ioc_grammars.md5.searchString(text)
    return _listify(md5s)


def parse_sha1s(text):
    """."""
    sha1s = ioc_grammars.sha1.searchString(text)
    return _listify(sha1s)


def parse_sha256s(text):
    """."""
    sha256s = ioc_grammars.sha256.searchString(text)
    return _listify(sha256s)


def parse_sha512s(text):
    """."""
    sha512s = ioc_grammars.sha512.searchString(text)
    return _listify(sha512s)


def parse_ssdeeps(text):
    """."""
    ssdeeps = ioc_grammars.ssdeep.searchString(text)
    return _listify(ssdeeps)


def parse_asns(text):
    """."""
    asns = ioc_grammars.asn.searchString(text)
    return _listify(asns)


def parse_cves(text):
    """."""
    cves = ioc_grammars.cve.searchString(text)
    return _listify(cves)


def parse_ipv4_cidrs(text: str) -> List:
    """."""
    cidrs = ioc_grammars.ipv4_cidr.searchString(text)
    return _listify(cidrs)


# def parse_ipv6_cidrs(text):
#     """."""
#     # TODO: implement
#     cidrs = ioc_grammars.ipv6_cidr.searchString(text)
#     return _listify(cidrs)


def parse_registry_key_paths(text):
    """."""
    parsed_registry_key_paths = ioc_grammars.registry_key_path.searchString(text)
    full_parsed_registry_key_paths = _listify(parsed_registry_key_paths)

    registry_key_paths = []
    for registry_key_path in full_parsed_registry_key_paths:
        # if there is a space in the last section of the parsed registry key path, remove it so that content after a registry key path is not also pulled in... this is a limitation of the grammar: it will not parse a registry key path with a space in the final section (the section after the final '\')
        if ' ' in registry_key_path.split('\\')[-1]:
            last_section = registry_key_path.split('\\')[-1]
            registry_key_path = registry_key_path.replace(last_section, last_section.split(' ')[0])
            registry_key_paths.append(registry_key_path)
        else:
            registry_key_paths.append(registry_key_path)

    return registry_key_paths


def parse_google_adsense_ids(text):
    """."""
    adsense_publisher_ids = ioc_grammars.google_adsense_publisher_id.searchString(text)
    return _listify(adsense_publisher_ids)


def parse_google_analytics_ids(text):
    """."""
    analytics_tracker_ids = ioc_grammars.google_analytics_tracker_id.searchString(text)
    return _listify(analytics_tracker_ids)


def parse_bitcoin_addresses(text):
    """."""
    bitcoin_addresses = ioc_grammars.bitcoin_address.searchString(text)
    return _listify(bitcoin_addresses)


def parse_monero_addresses(text):
    """."""
    monero_addresses = ioc_grammars.monero_address.searchString(text)
    return _listify(monero_addresses)


def parse_xmpp_addresses(text: str) -> List:
    """."""
    xmpp_addresses = ioc_grammars.xmpp_address.searchString(text)
    return _listify(xmpp_addresses)


def _remove_xmpp_local_part(xmpp_addresses: List, text: str) -> str:
    """Remove the local part of each xmpp_address from the text."""
    for address in xmpp_addresses:
        text = text.replace(address.split('@')[0] + '@', ' ')

    return text


def parse_mac_addresses(text):
    """."""
    mac_addresses = ioc_grammars.mac_address.searchString(text)
    return _listify(mac_addresses)


def parse_user_agents(text):
    """."""
    user_agents = ioc_grammars.user_agent.searchString(text)
    return _listify(user_agents)


def parse_file_paths(text):
    """."""
    file_paths = ioc_grammars.file_path.searchString(text)
    return _listify(file_paths)


def parse_phone_numbers(text):
    """."""
    phone_numbers = ioc_grammars.phone_number.searchString(text[::-1])
    return [phone_number[::-1] for phone_number in _listify(phone_numbers)]


def parse_pre_attack_tactics(text):
    """."""
    data = ioc_grammars.pre_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_pre_attack_techniques(text):
    """."""
    data = ioc_grammars.pre_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_mitigations(text):
    """."""
    data = ioc_grammars.enterprise_attack_mitigations_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_tactics(text):
    """."""
    data = ioc_grammars.enterprise_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_techniques(text):
    """."""
    data = ioc_grammars.enterprise_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_mitigations(text):
    """."""
    data = ioc_grammars.mobile_attack_mitigations_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_tactics(text):
    """."""
    data = ioc_grammars.mobile_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_techniques(text):
    """."""
    data = ioc_grammars.mobile_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_tlp_labels(text):
    """."""
    tlp_labels = ioc_grammars.tlp_label.searchString(text)
    return _listify(tlp_labels)


@click.command()
@click.argument('text', required=False)
@click.option('--no_url_domain_parsing', is_flag=True, help='Using this flag will not parse domain names from URLs')
@click.option('--no_parse_from_url_path', is_flag=True, help='Using this flag will not parse observables from URL paths')
@click.option(
    '--no_email_addr_domain_parsing',
    is_flag=True,
    help='Using this flag will not parse domain names from email addresses',
)
@click.option(
    '--no_cidr_address_parsing', is_flag=True, help='Using this flag will not parse IP addresses from CIDR ranges'
)
@click.option(
    '--no_xmpp_addr_domain_parsing',
    is_flag=True,
    help='Using this flag will not parse domain names from XMPP addresses',
)
@click.option('--no_urls_without_schemes', is_flag=True, help='Using this flag will not parse URLs without schemes')
@click.option('--no_import_hashes', is_flag=True, help='Using this flag will not parse import hashes')
@click.option('--no_authentihashes', is_flag=True, help='Using this flag will not parse authentihashes')
def cli_find_iocs(
    text,
    no_url_domain_parsing,
    no_parse_from_url_path,
    no_email_addr_domain_parsing,
    no_cidr_address_parsing,
    no_xmpp_addr_domain_parsing,
    no_urls_without_schemes,
    no_import_hashes,
    no_authentihashes,
):
    """CLI interface for parsing observables."""
    stdin_text = click.get_text_stream('stdin')

    # if there is stdin, use it
    if not text and stdin_text:
        text = '\n'.join([line for line in stdin_text])

    iocs = find_iocs(
        text,
        parse_domain_from_url=not no_url_domain_parsing,
        parse_from_url_path=not no_parse_from_url_path,
        parse_domain_from_email_address=not no_email_addr_domain_parsing,
        parse_address_from_cidr=not no_cidr_address_parsing,
        parse_domain_name_from_xmpp_address=not no_xmpp_addr_domain_parsing,
        parse_urls_without_scheme=not no_urls_without_schemes,
        parse_imphashes=not no_import_hashes,
        parse_authentihashes=not no_authentihashes,
    )
    ioc_string = json.dumps(iocs, indent=4, sort_keys=True)
    print(ioc_string)


def find_iocs(
    text: str,
    parse_domain_from_url: bool = True,
    parse_from_url_path: bool = True,
    parse_domain_from_email_address: bool = True,
    parse_address_from_cidr: bool = True,
    parse_domain_name_from_xmpp_address: bool = True,
    parse_urls_without_scheme: bool = True,
    parse_imphashes: bool = True,
    parse_authentihashes: bool = True,
) -> Dict[str, List]:
    """Find observables in the given text."""
    iocs = dict()

    text = _prepare_text(text)
    # keep a copy of the original text - some items should be parsed from the original text
    original_text = text

    # urls
    iocs['urls'] = parse_urls(text, parse_urls_without_scheme)
    if not parse_domain_from_url:
        text = _remove_items(iocs['urls'], text)

    if not parse_from_url_path:
        text = _remove_url_paths(iocs['urls'], text)

    # xmpp addresses
    iocs['xmpp_addresses'] = parse_xmpp_addresses(text)
    if not parse_domain_name_from_xmpp_address:
        text = _remove_items(iocs['xmpp_addresses'], text)
    # even if we want to parse domain names from the xmpp_address, we don't want them also being caught as email addresses so we'll remove everything before the `@`
    else:
        text = _remove_xmpp_local_part(iocs['xmpp_addresses'], text)

    # complete email addresses
    iocs['email_addresses_complete'] = parse_complete_email_addresses(text)
    # simple email addresses
    iocs['email_addresses'] = parse_email_addresses(text)
    if not parse_domain_from_email_address:
        text = _remove_items(iocs['email_addresses_complete'], text)
        text = _remove_items(iocs['email_addresses'], text)
    # after parsing the email addresses, we need to remove the '[IPv6:' bit from any of the email addresses so that ipv6 addresses are not extraneously parsed
    text = _remove_items(['[IPv6:'], text)

    # cidr ranges
    iocs['ipv4_cidrs'] = parse_ipv4_cidrs(text)
    if not parse_address_from_cidr:
        text = _remove_items(iocs['ipv4_cidrs'], text)
    # iocs['ipv6_cidrs'] = parse_ipv6_cidrs(text)
    # if not parse_address_from_cidr:
    #     text = _remove_items(iocs['ipv6_cidrs'], text)

    # file hashes
    if parse_imphashes:
        iocs['imphashes'] = parse_imphashes_(text)
        # remove the imphashes so they are not also parsed as md5s
        text = _remove_items(iocs['imphashes'], text)

    if parse_authentihashes:
        iocs['authentihashes'] = parse_authentihashes_(text)
        # remove the authentihashes so they are not also parsed as sha256s
        text = _remove_items(iocs['authentihashes'], text)

    with concurrent.futures.ProcessPoolExecutor() as executor:
        # domains
        iocs['domains'] = executor.submit(parse_domain_names, text).result()

        # ip addresses
        iocs['ipv4s'] = executor.submit(parse_ipv4_addresses, text).result()
        iocs['ipv6s'] = executor.submit(parse_ipv6_addresses, text).result()

        # file hashes
        iocs['sha512s'] = executor.submit(parse_sha512s, text).result()
        iocs['sha256s'] = executor.submit(parse_sha256s, text).result()
        iocs['sha1s'] = executor.submit(parse_sha1s, text).result()
        iocs['md5s'] = executor.submit(parse_md5s, text).result()
        iocs['ssdeeps'] = executor.submit(parse_ssdeeps, text).result()

        # misc
        iocs['asns'] = executor.submit(parse_asns, text).result()
        iocs['cves'] = executor.submit(parse_cves, [original_text]).result()
        iocs['registry_key_paths'] = executor.submit(parse_registry_key_paths, text).result()
        iocs['google_adsense_publisher_ids'] = executor.submit(parse_google_adsense_ids, text).result()
        iocs['google_analytics_tracker_ids'] = executor.submit(parse_google_analytics_ids, text).result()
        iocs['bitcoin_addresses'] = executor.submit(parse_bitcoin_addresses, text).result()
        iocs['monero_addresses'] = executor.submit(parse_monero_addresses, text).result()
        iocs['mac_addresses'] = executor.submit(parse_mac_addresses, text).result()
        iocs['user_agents'] = executor.submit(parse_user_agents, text).result()
        iocs['file_paths'] = executor.submit(parse_file_paths, text).result()
        iocs['phone_numbers'] = executor.submit(parse_phone_numbers, text).result()
        iocs['tlp_labels'] = executor.submit(parse_tlp_labels, original_text).result()

        iocs['attack_mitigations'] = {
            "enterprise": executor.submit(parse_enterprise_attack_mitigations, original_text).result(),
            "mobile": executor.submit(parse_mobile_attack_mitigations, original_text).result(),
        }
        iocs['attack_tactics'] = {
            "pre_attack": executor.submit(parse_pre_attack_tactics, original_text).result(),
            "enterprise": executor.submit(parse_enterprise_attack_tactics, original_text).result(),
            "mobile": executor.submit(parse_mobile_attack_tactics, original_text).result(),
        }
        iocs['attack_techniques'] = {
            "pre_attack": executor.submit(parse_pre_attack_techniques, original_text).result(),
            "enterprise": executor.submit(parse_enterprise_attack_techniques, original_text).result(),
            "mobile": executor.submit(parse_mobile_attack_techniques, original_text).result(),
        }

    return iocs



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

from click.testing import CliRunner

from ioc_finder import ioc_finder


def test_parse_cli_stdin():
    runner = CliRunner()
    result = runner.invoke(ioc_finder.cli_find_iocs, input="This is just an example.com https://example.org/test/bingo.php")
    assert result.exit_code == 0
    output = result.output.strip()
    assert 'example.org' in output
    assert 'example.com' in output
    assert 'https://example.org/test/bingo.php' in output


def test_ioc_parsing_cli():
    runner = CliRunner()
    result = runner.invoke(ioc_finder.cli_find_iocs, ["This is just an example.com https://example.org/test/bingo.php"])
    assert result.exit_code == 0
    output = result.output.strip()
    assert 'example.org' in output
    assert 'example.com' in output
    assert 'https://example.org/test/bingo.php' in output


def test_cli_without_domain_from_url_parsing():
    runner = CliRunner()
    result = runner.invoke(
        ioc_finder.cli_find_iocs,
        ["This is just an example.com https://example.org/test/bingo.php", "--no_url_domain_parsing"],
    )
    assert result.exit_code == 0
    print(result.output.strip())
    assert (
        result.output.strip()
        == """{
    "asns": [],
    "attack_mitigations": {
        "enterprise": [],
        "mobile": []
    },
    "attack_tactics": {
        "enterprise": [],
        "mobile": [],
        "pre_attack": []
    },
    "attack_techniques": {
        "enterprise": [],
        "mobile": [],
        "pre_attack": []
    },
    "authentihashes": [],
    "bitcoin_addresses": [],
    "cves": [],
    "domains": [
        "example.com"
    ],
    "email_addresses": [],
    "email_addresses_complete": [],
    "file_paths": [],
    "google_adsense_publisher_ids": [],
    "google_analytics_tracker_ids": [],
    "imphashes": [],
    "ipv4_cidrs": [],
    "ipv4s": [],
    "ipv6s": [],
    "mac_addresses": [],
    "md5s": [],
    "monero_addresses": [],
    "phone_numbers": [],
    "registry_key_paths": [],
    "sha1s": [],
    "sha256s": [],
    "sha512s": [],
    "ssdeeps": [],
    "tlp_labels": [],
    "urls": [
        "https://example.org/test/bingo.php"
    ],
    "user_agents": [],
    "xmpp_addresses": []
}"""
    )


def test_cli_parsing_urls_without_scheme():
    runner = CliRunner()
    result = runner.invoke(ioc_finder.cli_find_iocs, ["This is just an example.com example.org/test/bingo.php"])
    assert result.exit_code == 0
    print(result.output.strip())
    json_results = json.loads(result.output.strip())
    assert 'example.com' in json_results['domains']
    assert 'example.org' in json_results['domains']
    assert 'example.org/test/bingo.php' in json_results['urls']


def test_cli_disabling_parsing_urls_without_scheme():
    runner = CliRunner()
    result = runner.invoke(
        ioc_finder.cli_find_iocs,
        ["This is just an example.com example.org/test/bingo.php", "--no_urls_without_schemes"],
    )
    assert result.exit_code == 0
    print(result.output.strip())
    json_results = json.loads(result.output.strip())
    assert 'example.com' in json_results['domains']
    assert 'example.org' in json_results['domains']


def test_cli_disabling_import_hash_parsing():
    runner = CliRunner()
    result = runner.invoke(ioc_finder.cli_find_iocs, ["imphash 18ddf28a71089acdbab5038f58044c0a", "--no_import_hashes"])
    assert result.exit_code == 0
    json_results = json.loads(result.output.strip())
    assert '18ddf28a71089acdbab5038f58044c0a' in json_results['md5s']
    assert len(json_results['md5s']) == 1
    assert not json_results.get('imphashes')

    result = runner.invoke(ioc_finder.cli_find_iocs, ["imphash 18ddf28a71089acdbab5038f58044c0a"])
    assert result.exit_code == 0
    json_results = json.loads(result.output.strip())
    assert '18ddf28a71089acdbab5038f58044c0a' in json_results['imphashes']
    assert len(json_results['imphashes']) == 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import copy
import re

from pyparsing import alphas, alphanums, printables, nums, hexnums
from pyparsing import (
    CaselessLiteral,
    Combine,
    downcaseTokens,
    Empty,
    FollowedBy,
    Literal,
    NotAny,
    OneOrMore,
    Optional,
    Or,
    Regex,
    replaceWith,
    upcaseTokens,
    Word,
    WordEnd,
    WordStart,
    ZeroOrMore,
)

from data_lists import (
    pre_attack_tactics,
    pre_attack_techniques,
    enterprise_attack_mitigations,
    enterprise_attack_tactics,
    enterprise_attack_techniques,
    mobile_attack_mitigations,
    mobile_attack_tactics,
    mobile_attack_techniques,
    tlds,
    schemes,
)

alphanum_word_start = WordStart(wordChars=alphanums)
alphanum_word_end = WordEnd(wordChars=alphanums)

# the label definition ignores the fact that labels should not end in an hyphen
label = Word(initChars=alphanums + '_', bodyChars=alphanums + '-_', max=63)
domain_tld = Or(tlds)
domain_name = (
    alphanum_word_start
    + Combine(
        Combine(OneOrMore(label + ('.' + FollowedBy(Word(alphanums + '-_')))))('domain_labels') + domain_tld('tld')
    )
    + alphanum_word_end
).setParseAction(downcaseTokens)

ipv4_section = (
    Word(nums, asKeyword=True, max=3)
    .setParseAction(lambda x: str(int(x[0])))
    .addCondition(lambda tokens: int(tokens[0]) < 256)
)
# basically, the grammar below says: start any words that start with a '.' or a number; I want to match words that start with a '.' because this will fail later in the grammar and I do not want to match anything that start with a '.'
ipv4_address = (
    alphanum_word_start
    + WordStart('.' + nums)
    + Combine((ipv4_section + '.') * 3 + ipv4_section)
    + NotAny(Regex('\.\S'))
    + alphanum_word_end
)

hexadectet = Word(hexnums, min=1, max=4)
ipv6_address_full = alphanum_word_start + Combine((hexadectet + ":") * 7 + hexadectet)

ipv6_shortened_word_start = copy.deepcopy(alphanum_word_start)
# the condition on the end of this grammar is designed to make sure that any shortened ipv6 addresses have '::' in them
ipv6_address_shortened = Combine(OneOrMore(Or([hexadectet + Word(':'), Word(':')])) + hexadectet).addCondition(
    lambda tokens: tokens[0].count('::') > 0
)

ipv6_address = (
    Or([ipv6_address_full, ipv6_address_shortened]).addCondition(lambda tokens: tokens[0].count(':') > 1)
    + alphanum_word_end
)

complete_email_comment = Combine('(' + Word(printables.replace(')', '')) + ')')
# the complete_email_local_part grammar ignores the fact that characters like <<<(),:;<>@[\] >>> are possible in a quoted complete_email_local_part (and the double-quotes and backslash should be preceded by a backslash)
complete_email_local_part = Combine(
    Optional(complete_email_comment)('email_address_comment')
    + OneOrMore(Or([Word(alphanums + "!#$%&'*+-/=?^_`{|}~." + '"'), "\\@"]))
    + Optional(complete_email_comment)('email_address_comment')
)
complete_email_address = Combine(
    complete_email_local_part('email_address_local_part')
    + "@"
    + Or([domain_name, '[' + ipv4_address + ']', '[IPv6:' + ipv6_address + ']'])('email_address_domain')
)

email_local_part = Word(alphanums, bodyChars=alphanums + "+-_.").setParseAction(downcaseTokens)
email_address = alphanum_word_start + Combine(
    email_local_part('email_address_local_part')
    + "@"
    + Or([domain_name, '[' + ipv4_address + ']', '[IPv6:' + ipv6_address + ']'])('email_address_domain')
)

url_scheme = Or(schemes)
# todo: move the handling for port to the domain grammar - maybe?
port = Combine(':' + Word(nums))
url_authority = Combine(Or([complete_email_address, domain_name, ipv4_address, ipv6_address]) + Optional(port)('port'))
# NOTE: in the grammar below, we are excluding '?' and '#' so that the url's query strings and fragments are not parsed as part of a url path
# NOTE: in the grammar below, we are excluding '<' so that url in html are parsed properly (e.g. `<p>https://example.com/test/bingo.php</p>`)
url_path = Combine(OneOrMore(Word(printables, excludeChars='?#<') + Optional('/')))
url_query = Word(printables, excludeChars='#"\']')
url_fragment = Word(printables, excludeChars='?"\']')
url = alphanum_word_start + Combine(
    url_scheme('url_scheme')
    + '://'
    + url_authority('url_authority')
    + Optional(Combine('/' + Optional(url_path)))('url_path')
    + (Optional(Combine('?' + url_query)('url_query')) & Optional(Combine('#' + url_fragment)('url_fragment')))
)
scheme_less_url = alphanum_word_start + Combine(
    Or(
        [
            Combine(
                url_scheme('url_scheme')
                + '://'
                + url_authority('url_authority')
                + Optional(Combine('/' + Optional(url_path)))('url_path')
            ),
            Combine(url_authority('url_authority') + Combine('/' + Optional(url_path))('url_path')),
        ]
    )
    + (Optional(Combine('?' + url_query)('url_query')) & Optional(Combine('#' + url_fragment)('url_fragment')))
)

# this allows for matching file hashes preceeded with an 'x' or 'X' (https://github.com/fhightower/ioc-finder/issues/41)
file_hash_word_start = WordStart(wordChars=alphanums.replace('x', '').replace('X', ''))
md5 = file_hash_word_start + Word(hexnums, exact=32).setParseAction(downcaseTokens) + alphanum_word_end
imphash = Combine(
    Or(['imphash', 'import hash']) + Optional(Word(printables, excludeChars=alphanums)) + md5('hash'),
    joinString=' ',
    adjacent=False,
)
sha1 = file_hash_word_start + Word(hexnums, exact=40).setParseAction(downcaseTokens) + alphanum_word_end
sha256 = file_hash_word_start + Word(hexnums, exact=64).setParseAction(downcaseTokens) + alphanum_word_end
authentihash = Combine(
    Or(['authentihash']) + Optional(Word(printables, excludeChars=alphanums)) + sha256('hash'),
    joinString=' ',
    adjacent=False,
)
sha512 = file_hash_word_start + Word(hexnums, exact=128).setParseAction(downcaseTokens) + alphanum_word_end

year = Word('12') + Word(nums, exact=3)
cve = (
    alphanum_word_start
    + Combine(
        Or(['cve', 'CVE']).setParseAction(replaceWith('CVE'))
        + Word('- ').setParseAction(replaceWith('-'))
        + year('year')
        + Word('-')
        + Word(nums, min=4)('cve_id')
    )
    + alphanum_word_end
)

asn = (
    alphanum_word_start
    + Combine(
        Or(['as', 'AS']).setParseAction(replaceWith('AS'))
        + Optional(Word('nN ')).setParseAction(replaceWith('N'))
        + Word(nums)('as_number')
    )
    + alphanum_word_end
)

# todo: implement ipv6 cidr ranges
ipv4_cidr = (
    alphanum_word_start
    + Combine(ipv4_address('cidr_address') + '/' + Word(nums, max=2)('cidr_bit_range'))
    + alphanum_word_end
)

root_key_list = [
    'HKEY_LOCAL_MACHINE',
    'HKLM',
    'HKEY_CURRENT_CONFIG',
    'HKCC',
    'HKEY_CLASSES_ROOT',
    'HKCR',
    'HKEY_CURRENT_USER',
    'HKCU',
    'HKEY_USERS',
    'HKU',
    'HKEY_PERFORMANCE_DATA',
    'HKEY_DYN_DATA',
]

root_key = Or(root_key_list)


def hasMultipleConsecutiveSpaces(string):
    """Return True if the given string has multiple, consecutive spaces."""
    return re.match('  +', string)


def hasBothOrNeitherAngleBrackets(string):
    """Make sure a string either has both '<' and '>' or neither of those angle brackets."""
    left_angle_bracket_in_string = '<' in string
    right_angle_bracket_in_string = '>' in string

    # if the string has both brackets...
    if left_angle_bracket_in_string and right_angle_bracket_in_string:
        return True
    # if the string has only one bracket...
    elif left_angle_bracket_in_string or right_angle_bracket_in_string:
        return False
    # if the string has neither of the brackets...
    else:
        return True


registry_key_subpath_section = Combine(
    Word('\\')
    + Optional(Word('<'))
    + Word(alphanums)
    + ZeroOrMore(
        # registry key paths may contain a file extension which requires that we capture registry key path sections with a period (e.g. `notepad.exe`)
        Optional(Word('.', max=1))
        # the registry key path section can contain any alphanum text (including spaces) as long as the text is not one of the registry key path root keys and as long as there are not multiple, consecutive spaces
        + Word(alphanums + ' ').addCondition(
            lambda tokens: tokens[0] not in root_key_list and not hasMultipleConsecutiveSpaces(tokens[0])
        )
    )
    + Optional(Word('>'))
).addCondition(lambda tokens: hasBothOrNeitherAngleBrackets(tokens[0]))
registry_key_subpath = OneOrMore(registry_key_subpath_section)
registry_key_path = (
    alphanum_word_start
    + Combine(
        Optional('<').setParseAction(replaceWith(''))
        + root_key('registry_key_root')
        + Optional('>').setParseAction(replaceWith(''))
        + registry_key_subpath('registry_key_subpath')
    )
    + alphanum_word_end
)

# see https://support.google.com/adsense/answer/2923881?hl=en
google_adsense_publisher_id = (
    alphanum_word_start
    + Combine(Or(['pub-', 'PUB-']) + Word(nums, exact=16)).setParseAction(downcaseTokens)
    + alphanum_word_end
)

# see https://support.google.com/analytics/answer/7372977?hl=en
google_analytics_tracker_id = (
    alphanum_word_start
    + Combine(
        Or(['UA-', 'ua-']) + Word(nums, min=6)('account_number') + '-' + Word(nums)('property_number')
    ).setParseAction(upcaseTokens)
    + alphanum_word_end
)

# see https://en.bitcoin.it/wiki/Address (and https://github.com/bitcoin/bips/blob/master/bip-0173.mediawiki#segwit-address-format for more info on Bech32 addresses)
bitcoin_address = (
    alphanum_word_start
    + Or(
        [
            Combine('1' + Word(alphanums, min=25, max=34)),
            Combine('3' + Word(alphanums, min=25, max=34)),
            Combine('bc1' + Word(alphanums, min=11, max=71)),
        ]
    )
    + alphanum_word_end
)

monero_address = alphanum_word_start + Regex('4[0-9AB][1-9A-HJ-NP-Za-km-z]{93}') + alphanum_word_end

# see https://github.com/fhightower/ioc-finder/issues/18
xmpp_address = alphanum_word_start + Combine(
    email_local_part('email_address_local_part') + "@" + domain_name('jabber_address_domain')
).addCondition(lambda tokens: 'jabber' in tokens[0].split('@')[-1] or 'xmpp' in tokens[0].split('@')[-1])

# the mac address grammar was developed from https://en.wikipedia.org/wiki/MAC_address#Notational_conventions
# handles xx:xx:xx:xx:xx:xx or xx-xx-xx-xx-xx-xx
mac_address_16_bit_section = Combine((Word(hexnums, exact=2) + Or(['-', ':'])) * 5 + Word(hexnums, exact=2))
# handles xxxx.xxxx.xxxx
mac_address_32_bit_section = Combine((Word(hexnums, exact=4) + '.') * 2 + Word(hexnums, exact=4))
mac_address = alphanum_word_start + Or([mac_address_16_bit_section, mac_address_32_bit_section]) + alphanum_word_end

# the structure of an ssdeep hash is: chunksize:chunk:double_chunk
# we add a condition to the ssdeep grammar to make sure that the second section of the grammar (the chunk) is at least as big if not bigger than the third section (the double_chunk)
ssdeep = alphanum_word_start + Combine(
    Word(nums) + ':' + Word(alphanums + '/+', min=3) + ':' + Word(alphanums + '/+', min=3)
).addCondition(lambda tokens: len(tokens[0].split(':')[1]) >= len(tokens[0].split(':')[2]))

user_agent_platform_version = Regex('[0-9]+(\.[0-9]*)*')
user_agent_start = Combine(Regex('[Mm]ozilla/') + user_agent_platform_version)
user_agent_details = Regex('\(.+?\)')
user_agent_platform = Combine(
    alphanum_word_start
    + Regex('[a-zA-Z]{2,}/?').addCondition(lambda tokens: tokens[0].lower().strip('/') != 'mozilla')
    + Optional(user_agent_platform_version)
)
user_agent = Combine(
    user_agent_start + user_agent_details + ZeroOrMore(user_agent_platform + Optional(user_agent_details)),
    joinString=' ',
    adjacent=False,
)

# https://github.com/fhightower/ioc-finder/issues/13
# TODO: improve the windows_file_path grammar - it is pretty naive right now... the file_ending is very basic and it would be nice to have a list of common file endings, the windows_file_path grammar assumes that a path will not have a '.' in it (other than in the file name at the end), and the windows_file_path grammar assumes that the path will have a file name at the end (it will not match directory paths well)
file_ending = Word(alphas, max=5)
windows_file_path = alphanum_word_start + Combine(
    Word(alphanums, exact=1) + ':' + Word(printables.replace('.', '') + ' ') + '.' + file_ending
)

# we need to add '/' and '~' to the alphanum_word_start so that the grammar will match words starting with '/' and '~'
# we add ':' to the alphanum_word_start because we want to avoid parsing urls are file paths (e.g. "//twitter.com" from "https://twitter.com/")
unix_file_path_wordstart = copy.deepcopy(alphanum_word_start)
unix_file_path_wordstart.wordChars.add(':')
unix_file_path_wordstart.wordChars.add('/')
unix_file_path_wordstart.wordChars.add('~')

unix_file_path = unix_file_path_wordstart + Combine(
    Or(['~', '/']) + Word(printables.replace('.', '') + ' ') + '.' + file_ending
).addCondition(lambda tokens: '//' not in tokens[0])
file_path = Or([windows_file_path, unix_file_path]) + alphanum_word_end

# be aware that the phone_number grammar assumes that the text being sent to it has been reversed
phone_number_connector = Word(' .-', max=3)
phone_number_format_1 = Combine(
    Word(nums, exact=4)
    + phone_number_connector
    + Word(nums, exact=3)
    + Optional(phone_number_connector + Optional(')') + Word(nums) + Optional('('))
)

phone_number = Or([phone_number_format_1])

attack_sub_technique = Literal('.') + Word(nums, exact=3)
pre_attack_tactics_grammar = (
    alphanum_word_start
    + Or([CaselessLiteral(i) for i in pre_attack_tactics]).setParseAction(upcaseTokens)
    + alphanum_word_end
)
pre_attack_techniques_grammar = (
    alphanum_word_start
    + Combine(
        Or([CaselessLiteral(i) for i in pre_attack_techniques]).setParseAction(upcaseTokens)
        + Optional(attack_sub_technique)
    )
    + alphanum_word_end
)

enterprise_attack_mitigations_grammar = (
    alphanum_word_start + Or([CaselessLiteral(i) for i in enterprise_attack_mitigations]) + alphanum_word_end
)
enterprise_attack_tactics_grammar = (
    alphanum_word_start
    + Or([CaselessLiteral(i) for i in enterprise_attack_tactics]).setParseAction(upcaseTokens)
    + alphanum_word_end
)
enterprise_attack_techniques_grammar = (
    alphanum_word_start
    + Combine(
        Or([CaselessLiteral(i) for i in enterprise_attack_techniques]).setParseAction(upcaseTokens)
        + Optional(attack_sub_technique)
    )
    + alphanum_word_end
)

mobile_attack_mitigations_grammar = (
    alphanum_word_start + Or([CaselessLiteral(i) for i in mobile_attack_mitigations]) + alphanum_word_end
)
mobile_attack_tactics_grammar = (
    alphanum_word_start
    + Or([CaselessLiteral(i) for i in mobile_attack_tactics]).setParseAction(upcaseTokens)
    + alphanum_word_end
)
mobile_attack_techniques_grammar = (
    alphanum_word_start
    + Combine(
        Or([CaselessLiteral(i) for i in mobile_attack_techniques]).setParseAction(upcaseTokens)
        + Optional(attack_sub_technique)
    )
    + alphanum_word_end
)

tlp_colors_list = [CaselessLiteral('red'), CaselessLiteral('amber'), CaselessLiteral('green'), CaselessLiteral('white')]
tlp_colors = Or(tlp_colors_list)

tlp_label = Combine(
    CaselessLiteral('tlp')
    + Or([Literal(':'), Literal('-'), Literal(' '), Empty()]).setParseAction(lambda x: ':')
    + tlp_colors
).setParseAction(upcaseTokens)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ioc_finder.ioc_grammars import hasMultipleConsecutiveSpaces, hasBothOrNeitherAngleBrackets


def test_hasBothOrNeitherAngleBrackets_1():
    assert hasBothOrNeitherAngleBrackets('<>')
    assert hasBothOrNeitherAngleBrackets('<foo>')
    assert hasBothOrNeitherAngleBrackets('< foo >')
    assert hasBothOrNeitherAngleBrackets('foo')

    assert not hasBothOrNeitherAngleBrackets('<')
    assert not hasBothOrNeitherAngleBrackets('<foo')
    assert not hasBothOrNeitherAngleBrackets('foo<')
    assert not hasBothOrNeitherAngleBrackets('<foo<')
    assert not hasBothOrNeitherAngleBrackets('>')
    assert not hasBothOrNeitherAngleBrackets('>foo')
    assert not hasBothOrNeitherAngleBrackets('foo>')
    assert not hasBothOrNeitherAngleBrackets('>foo>')


def test_hasMultipleConsecutiveSpaces_1():
    assert not hasMultipleConsecutiveSpaces('')
    assert not hasMultipleConsecutiveSpaces(' ')
    assert hasMultipleConsecutiveSpaces('  ')
    assert hasMultipleConsecutiveSpaces('   ')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ioc_finder import find_iocs


def test_attack_enterprise_mitigations_1():
    s = '''Mitigations: 41
ID  Name    Description
M1036   Account Use Policies    
Configure features related to account use like login attempt lockouts, specific login times, etc.

M1015   Active Directory Configuration  
Configur'''
    results = find_iocs(s)
    assert list(results['attack_mitigations'].keys()) == ['enterprise', 'mobile']
    assert results['attack_mitigations']['mobile'] == []
    assert 'M1036' in results['attack_mitigations']['enterprise']
    assert 'M1015' in results['attack_mitigations']['enterprise']


def test_attack_mobile_mitigations_1():
    s = ''' Name    Description
M1013   Application Developer Guidance  
This mitigation describes any guidance or training given to developers of applications to avoid introducing security weaknesses that an adversary may be able to take advantage of.

M1005   Application Vetting 
Enterprises can vet applications for exploitable vulnerabilities or unwanted (privacy-invasive or malicious) behaviors. Enterprises can inspect appl'''
    results = find_iocs(s)
    assert list(results['attack_mitigations'].keys()) == ['enterprise', 'mobile']
    assert results['attack_mitigations']['enterprise'] == ['M1013']
    assert 'M1013' in results['attack_mitigations']['mobile']
    assert 'M1005' in results['attack_mitigations']['mobile']


def test_attack_techniques():
    s = """
ID  Name    Description
T1329   Acquire and/or use 3rd party infrastructure services    
A wide variety of cloud, virtual private services, hosting, compute, and storage solutions are available. Additionally botnets are available for rent or purchase. Use of these solutions allow an adversary to stage, launch, and execute an attack from infrastructure that does not physically tie back to them and can be rapidly provisioned, modified, and shut down.

T1307   Acquire and/or use 3rd party infrastructure services    
A wide variety of cloud, virtual private services, hosting, compute, and storage solutions are available. Additionally botnets are available for rent or purchase. Use of these solutions allow an adversary to stage, launch, and execute an attack from infrastructure that does not physically tie back to them and can be rapidly provisioned, modified, and shut down.

T1308"""
    results = find_iocs(s)
    print(results)
    assert len(results['attack_techniques']) == 3
    assert 'T1329' in results['attack_techniques']['pre_attack']
    assert 'T1307' in results['attack_techniques']['pre_attack']
    assert 'T1308' in results['attack_techniques']['pre_attack']


def test_attack_tactics():
    s = """
    ID  Name    Description
    TA0012  Priority Definition Planning    Priority definition planning consists of the process of determining the set of Key Intelligence Topics (KIT) or Key Intelligence Questions (KIQ) required for meeting key strategic, operational, or tactical goals. Leadership outlines the priority definition (may be considered a goal) around which the adversary designs target selection and a plan to achieve. An analyst may outline the priority definition when in the course of determining gaps in existing KITs or KIQs.
    TA0013  Priority Definition Direction   Priority definition direction consists of the process of collecting and assigning requirements for meeting Key Intelligence Topics (KIT) or Key Intelligence Questions (KIQ) as determined by leadership.
    TA0014  Targ"""
    results = find_iocs(s)
    assert len(results['attack_tactics']) == 3
    assert 'TA0012' in results['attack_tactics']['pre_attack']
    assert 'TA0013' in results['attack_tactics']['pre_attack']
    assert 'TA0014' in results['attack_tactics']['pre_attack']


def test_attack_techniques_edge_cases():
    # make sure attack techniques preceded by some alpha-num. character are not parsed
    s = """FOOT1329"""
    results = find_iocs(s)
    assert results['attack_techniques']['pre_attack'] == []
    assert results['attack_techniques']['enterprise'] == []
    assert results['attack_techniques']['mobile'] == []

    # make sure attack techniques postceded by some alpha-num. character are not parsed
    s = """T1329FUN"""
    results = find_iocs(s)
    assert results['attack_techniques']['pre_attack'] == []
    assert results['attack_techniques']['enterprise'] == []
    assert results['attack_techniques']['mobile'] == []

    # make sure attack techniques preceeded by some alpha-num. character are not parsed
    s = """foot1329"""
    results = find_iocs(s)
    assert results['attack_techniques']['pre_attack'] == []
    assert results['attack_techniques']['enterprise'] == []
    assert results['attack_techniques']['mobile'] == []

    # test lower-case matching
    s = """t1329"""
    results = find_iocs(s)
    print(results)
    assert results['attack_techniques']['pre_attack'] == ['T1329']

    s = 'T1156'
    results = find_iocs(s)
    assert results['attack_techniques']['enterprise'] == ['T1156']

    s = "https://attack.mitre.org/techniques/T1156/"
    results = find_iocs(s)
    assert results['attack_techniques']['enterprise'] == ['T1156']


def test_attack_tactics_edge_cases():
    # make sure attack tactics preceeded by some alpha-num. character are not parsed
    s = """FOOTA0001"""
    results = find_iocs(s)
    assert results['attack_tactics']['pre_attack'] == []
    assert results['attack_tactics']['enterprise'] == []
    assert results['attack_tactics']['mobile'] == []

    s = """AT0001"""
    results = find_iocs(s)
    assert results['attack_tactics']['pre_attack'] == []
    assert results['attack_tactics']['enterprise'] == []
    assert results['attack_tactics']['mobile'] == []

    # make sure attack tactics postceded by some alpha-num. character are not parsed
    s = """TA0001FUN"""
    results = find_iocs(s)
    assert results['attack_tactics']['pre_attack'] == []
    assert results['attack_tactics']['enterprise'] == []
    assert results['attack_tactics']['mobile'] == []

    # make sure attack tactics preceeded by some alpha-num. character are not parsed
    s = """foota0001"""
    results = find_iocs(s)
    assert results['attack_tactics']['pre_attack'] == []
    assert results['attack_tactics']['enterprise'] == []
    assert results['attack_tactics']['mobile'] == []

    # test lower-case matching
    s = """ta0001"""
    results = find_iocs(s)
    assert results['attack_tactics']['enterprise'] == ['TA0001']

    s = "https://attack.mitre.org/tactics/TA0001/"
    results = find_iocs(s)
    assert results['attack_tactics']['enterprise'] == ['TA0001']


def test_subtechnique_parsing():
    s = 'T1546.004'
    results = find_iocs(s)
    assert results['attack_techniques']['enterprise'] == ['T1546.004']

    s = 'T1546.004'
    results = find_iocs(s)
    assert results['attack_techniques']['enterprise'] == ['T1546.004']

    s = 'T1156.0012'
    results = find_iocs(s)
    assert results['attack_techniques']['enterprise'] == []
